<?php

/**
 * @file
 * The Forms used by the plugin manager.
 */

define('PLUGIN_MANAGER_PAGE_INTRO', 0);
define('PLUGIN_MANAGER_PAGE_VERSION_SELECT', 1);
define('PLUGIN_MANAGER_PAGE_SUMMARY', 2);

/**
 * Choose between the steps in the install form.
 */
function plugin_manager_install_form($form_state) {
  // Some forms can be skipped immediately.
  $return_value = FALSE;
  while($return_value === FALSE) {
    if (!isset($form_state['storage']['page'])) {
      $form_state['storage']['page'] = PLUGIN_MANAGER_PAGE_INTRO;
    }

    switch($form_state['storage']['page']) {
      case PLUGIN_MANAGER_PAGE_INTRO:
        $return_value = plugin_manager_install_intro($form_state);
        break;

      case PLUGIN_MANAGER_PAGE_VERSION_SELECT:
        $return_value = plugin_manager_install_version_select($form_state);
        break;

      case PLUGIN_MANAGER_PAGE_SUMMARY:
        $return_value = plugin_manager_install_summary($form_state);
        break;

      /* The first page is the default.  It should give a brief overview. */
      default:
        $form_state['storage']['page'] = PLUGIN_MANAGER_PAGE_INTRO;
        break;
    }
  }
  return $return_value;
}

/**
 * Allow the user to choose the version of the plugin to install.
 */
function plugin_manager_install_intro($form_state) {
  $queue = plugin_manager_queue_get();
  if (count($queue) == 0)  {
    drupal_set_message(t("There's no need to be in a rush.  Before we can install any plugins we need to select them. ") . l(t('You can add some here.'), 'admin/plugin_manager'), 'warning');
    return array();
  }

  $form['instructions'] = array(
    '#type' => 'markup',
    '#value' => "<p>" . t("Let's get started.  Before I can install the @plugins you selected, I have to know what version of each to use.  I can choose the recommended one for you, or if you would prefer, you can choose one in particular.  So, what will it be?", array('@plugins' => format_plural(count($queue), 'plugin', 'plugins'))) . "</p>",
  );
  $form['submit'] = array(
    '#name' => 'automatic',
    '#type' => 'submit',
    '#value' => t('Choose for me'),
  );
  $form['choose'] = array(
    '#name' => 'manual',
    '#type' => 'submit',
    '#value' => t('Let me choose...'),
  );
  $form['#submit'][] = 'plugin_manager_install_intro_submit';
  return $form;
}

function plugin_manager_install_intro_submit($form, &$form_state) {
  // @Todo: Add chmod option also.
  if ($form_state['clicked_button']['#name'] == 'manual') {
    $form_state['storage']['page'] = PLUGIN_MANAGER_PAGE_VERSION_SELECT;
  }
  else {
    // Since the user chose automatic, we need to figure out which version is best.  Afterwards we can go straight to the summary page.
    $queue = plugin_manager_queue_get();
    foreach($queue AS $short_name => $project) {
      $field = $project['project']->get_fields();
      $releases = array_keys($project['project']->get_releases());
      $form_state['storage']['version_list'][$short_name] = array_shift($releases);
    }
    $form_state['storage']['page'] = PLUGIN_MANAGER_PAGE_SUMMARY;
  }
}

/**
 * Allow the user to choose the version of the plugin to install.
 */
function plugin_manager_install_version_select($form_state) {
  $queue = plugin_manager_queue_get();
  $form['markup'] = array(
    '#type' => 'markup',
    '#value' => '<p>' . t('The most recent recommended version of each project is currently selected.') . '</p>',
  );

  $form['version_list'] = array(
    '#tree' => TRUE,
  );
  foreach($queue AS $short_name => $project) {
    $field = $project['project']->get_fields();
    $releases = $project['project']->get_releases();
    
    $form['version_list'][$short_name] = array(
      '#title' => $field['title'],
      '#type' => 'select',
      '#options' => array_combine(array_keys($releases), array_keys($releases)), 
    );
  }

  // @Todo: Check permissions.  Change text to 'Grant permissions' or something if the server cannot write.
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Install',
  );
  $form['#submit'][] = 'plugin_manager_install_version_select_submit';
  return $form;
}

/**
 *
 */
function plugin_manager_install_version_select_submit($form, &$form_state) {
  $form_state['storage']['version_list'] = $form_state['values']['version_list'];
  $form_state['storage']['page'] = PLUGIN_MANAGER_PAGE_SUMMARY;
}

/**
 * Allow the user to choose the version of the plugin to install.
 */
function plugin_manager_install_summary($form_state) {
  $queue = plugin_manager_queue_get();
  $form['markup'] = array(
    '#type' => 'markup',
    '#value' => '<p>'. t('Before the actual installation, please check this list one more time.') .'</p>',
  );
  foreach($queue AS $project => $info) {
    $fields = $info['project']->get_fields();
    $form['listing'][$project] = array(
      '#type' => 'markup',
      '#value' => '<p>'. $fields['title'] .' ('. $form_state['storage']['version_list'][$project] .')</p>',
    );
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Install',
  );
  $form['#submit'][] = 'plugin_manager_install_summary_submit';
  return $form;
}

/**
 *
 */
function plugin_manager_install_summary_submit($form, &$form_state) {
  // Otherwise I keep ending up on last page of the installer again.
  unset($form_state['storage']['page']);

  // Set the desired versions and then process the queue.
  $queue = plugin_manager_queue_get();
  foreach($form_state['storage']['version_list'] AS $short_name => $version) {
    $project = $queue[$short_name]['project'];
    plugin_manager_set_version($project, $version);
  }

  plugin_manager_process_queue();
}

/**
 * This is the main function for the Ajax Search page
 * of Plugin Manager.
 */
function plugin_manager_search_form(){
  drupal_add_js(drupal_get_path("module", "plugin_manager") ."/plugin_manager_search.js", 'module');
  drupal_add_css(drupal_get_path("module", "plugin_manager") ."/plugin_manager_search.css", 'module');

  // Are there backends to actually process input?
  $backends = module_implements('pm_process');
  drupal_add_js(array('plugin_manager' => array('backends' => !empty($backends))), 'setting');

  // Generate the search form.
  $form['input'] = array('#type' => 'textfield',);
  $form['search'] = array('#value' => t('Search'), '#type' => 'submit',);
  $form['queued'] = array('#value' => t('Queue'), '#type' => 'submit',);
  $form['process'] = array('#value' => t('Process Queue'), '#type' => 'submit',);
  $form['results'] = array('#type' => 'fieldset', '#title' => t('Results'), '#collapsible' => TRUE,);
  return $form;
}

/**
 *
 */
function plugin_manager_search_form_submit($form, &$form_values) {
  // If the process button was pressed then forward to the installer.
  if ($form_values['values']['op'] == $form_values['values']['process']) {
    drupal_goto('admin/build/plugin_manager/install');
  }
}
